import typer
from typing import Optional
from proximatic import Proximatic


app = typer.Typer()

proximatic = Proximatic()


@app.callback()
def callback():
    """
    Proximatic CLI\n
    Manage your Proximatic configuration files.
    """


@app.command()
def domain_list():
    """Returns a list of configured domains."""
    domains = proximatic.domain_list()["domains"]
    typer.echo("Domains:\n--------")
    for domain in domains:
        for name, url in domain.items():
            typer.echo(f"{name}.example.com <=proxy=> {url}")


@app.command()
def domain_update(
    subdomain: str = typer.Option(..., prompt="Subdomain"),
    url: str = typer.Option(..., prompt="URL"),
):
    """Creates or replaces a domain configuration .yml file."""
    response = proximatic.domain_update(subdomain=subdomain, url=url)
    if response["result"] == "success":
        typer.echo("Domain updated successfully.")
        typer.echo(f"{subdomain}.{proximatic.proximatic_fqdn} => {url}")
    else:
        raise typer.Exit(code=1)


@app.command()
def domain_delete(
    subdomain: str = typer.Option(..., prompt="Subdomain"),
):
    response = proximatic.domain_delete(subdomain=subdomain)
    if response["result"] == "success":
        typer.echo(f"Domain {subdomain} was deleted successfully.")
    else:
        typer.echo(response["result"])
        raise typer.Exit(code=1)



import os
import glob
import yaml
import requests
from jinja2 import Environment, PackageLoader


class Proximatic:
    def __init__(
        self, yml_dir: str = "/data/traefik/conf/", proximatic_fqdn: str = None
    ):
        self.yml_dir = yml_dir

        if proximatic_fqdn:
            self.proximatic_fqdn = proximatic_fqdn
        elif os.getenv("PROXIMATIC_FQDN"):
            self.proximatic_fqdn = os.getenv("PROXIMATIC_FQDN")
        else:
            self.proximatic_fqdn = "example.com"

    def set_fqdn(self, fqdn: str):
        self.proximatic_fqdn = fqdn

    def get_fqdn(self):
        return self.proximatic_fqdn

    def domain_list(self):
        response = {"domains": []}
        files = glob.glob(self.yml_dir + "*.yml")
        for filename in files:
            with open(filename, "r") as yml_stream:
                config = yaml.safe_load(yml_stream)
                for service, values in config["http"]["services"].items():
                    response["domains"].append(
                        {service: values["loadBalancer"]["servers"][0]["url"]}
                    )
        return response

    def domain_update(self, subdomain: str, url: str):
        # Check the URL for validity by visiting it expecting a 200 response code from its server.
        try:
            result = requests.get(url).status_code
            if result != 200:
                return {
                    "Error": "Invalid URL"
                }  # @todo define some reusable error response payloads.
        except Exception as e:
            return {"error": "Invalid URL", "msg": str(e)}

        # Load Jinja2 template engine.
        env = Environment(loader=PackageLoader('proximatic', 'templates'))
        template = env.get_template("domain.j2.yml")
        # Use template to generate a string of YAML containing valid Traefik config.
        yml_string = template.render(
            subdomain=subdomain,
            proximatic_fqdn=self.proximatic_fqdn,
            url=url,
        )
        # Write the YAML to a .yml file named after the subdomain.
        yml_file_path = self.yml_dir + subdomain + ".yml"
        yml_file = open(yml_file_path, "wt")
        lines_written = yml_file.write(yml_string)
        yml_file.close()
        response = {"result": "success", "subdomain": subdomain, "url": url}
        return response

    def domain_delete(self, subdomain: str):
        yml_file_path = self.yml_dir + subdomain + ".yml"
        response = {
            "result": "success",
            "subdomain": subdomain,
        }
        if os.path.exists(yml_file_path):
            os.remove(yml_file_path)
        else:
            response["result"] = "File not found"
        return response



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['proximatic']

package_data = \
{'': ['*'], 'proximatic': ['templates/*']}

install_requires = \
['Jinja2>=2.11.3,<3.0.0',
 'PyYAML>=5.4.1,<6.0.0',
 'requests>=2.25.1,<3.0.0',
 'typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['proximatic = proximatic.main:app']}

setup_kwargs = {
    'name': 'proximatic',
    'version': '0.1.2',
    'description': 'Python API for managing Proximatic configuration files.',
    'long_description': '# Proximatic\n\nPython API for managing Proximatic configuration files.\n\nWhen installed, the `proximatic` command provides a CLI for managing Proximatic configuration.\n\nThis Python package provides the core for the Proximatic system.\n\n## Installation\n\n```bash\npip install proximatic\n```\n\n## Usage\n\n### Command Line Interface (CLI)\n\nOpen a Terminal and type:\n\n```bash\nproximatic\n```\n\nUse `proximatic --help` for available commands and options.\n\n### Python API programmatic interface\n\n```python\nimport proximatic\n\n# List existing domains.\nmanager = proximatic.DomainsManager(yml_dir = \'/path/to/your/data/traefik/conf/\')\nresult = manager.domain_list()\nfor domain in result[\'domains\']:\n    for name, url in domain.items():\n        print(f"{name} <=proxy=> {url}")\n\n# Create new or update existing domain.\n\nmanager.set_fqdn(\'yourdomain.com\') # change to your domain\nresult = manager.domain_update(subdomain = \'mysubdomain\', url = \'https://news.ycombinator.com\')\n\n# See your newly created domain.\nresult = manager.domain_list()\nfor domain in result[\'domains\']:\n    for name, url in domain.items():\n        print(f"{name} <=proxy=> {url}")\n\n# Delete the domain.\nresult = manager.domain_delete(\'mysubdomain\')\nprint(result)\n\n# See that it is gone.\nresult = manager.domain_list()\nfor domain in result[\'domains\']:\n    for name, url in domain.items():\n        print(f"{name} <=proxy=> {url}")\n```\n\n## License\n\nThe MIT License (MIT)\n\n## Author\n\nLink Swanson (LunkRat)',
    'author': 'Link Swanson',
    'author_email': 'link@swanson.link',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
